@@ -21,8 +21,8 @@ import argparse
 import os
 import copy
 import inspect
+import time
 from concurrent import futures
-from datetime import datetime,timedelta
 from typing import List, Callable
 from bittensor._threadpool import prioritythreadpool
 
@@ -34,7 +34,15 @@ import bittensor
 from . import axon_impl
 
 class axon:
-    """ Create and init Axon, whcih services Forward and Backward requests from other neurons.
+    """ The factor class for bittensor.Axon object
+    The Axon acts a grpc server for the bittensor network and allows for communication between neurons.
+    By default, the grpc server follows the bittensor protocol and transports forward and backwards requests
+    between validators and servers. 
+    
+    Examples:: 
+            >>> axon = bittensor.axon(config=config)
+            >>> subtensor = bittensor.subtensor(network='nakamoto')
+            >>> axon.serve(subtensor=subtensor)
     """
 
     def __new__(
@@ -43,10 +51,11 @@ class axon:
             wallet: 'bittensor.Wallet' = None,
             forward_text: 'Callable' = None,
             backward_text: 'Callable' = None,
-            forward_image: 'Callable' = None,
-            backward_image: 'Callable' = None,
-            forward_tensor: 'Callable' = None,
-            backward_tensor: 'Callable' = None,
+            synapse_last_hidden: 'Callable' = None,
+            synapse_causal_lm: 'Callable' = None,
+            synapse_causal_lm_next: 'Callable' = None,
+            synapse_seq_2_seq: 'Callable' = None,
+            synapse_checks: 'Callable' = None,
             thread_pool: 'futures.ThreadPoolExecutor' = None,
             server: 'grpc._Server' = None,
             port: int = None,
@@ -57,6 +66,7 @@ class axon:
             priority: 'Callable' = None,
             forward_timeout: int = None,
             backward_timeout: int = None,
+            compression: str = None,
         ) -> 'bittensor.Axon':
         r""" Creates a new bittensor.Axon object from passed arguments.
             Args:
@@ -68,14 +78,16 @@ class axon:
                     function which is called on forward text requests.
                 backward_text (:obj:`callable`, `optional`):
                     function which is called on backward text requests.
-                forward_image (:obj:`callable`, `optional`):
-                    function which is called on forward image requests.
-                backward_image (:obj:`callable`, `optional`):
-                    function which is called on backward image requests.
-                forward_tensor (:obj:`callable`, `optional`):
-                    function which is called on forward tensor requests.
-                backward_tensor (:obj:`callable`, `optional`):
-                    function which is called on backward tensor requests.
+                synapse_last_hidden (:obj:`callable`, `optional`):
+                    function which is called by the last hidden synapse
+                synapse_causal_lm (:obj:`callable`, `optional`):
+                    function which is called by the causal lm synapse
+                synapse_causal_lm_next (:obj:`callable`, `optional`):
+                    function which is called by the TextCausalLMNext synapse
+                synapse_seq_2_seq (:obj:`callable`, `optional`):
+                    function which is called by the seq2seq synapse   
+                synapse_checks (:obj:`callable`, 'optional'):
+                    function which is called before each synapse to check for stake        
                 thread_pool (:obj:`ThreadPoolExecutor`, `optional`):
                     Threadpool used for processing server queries.
                 server (:obj:`grpc._Server`, `required`):
@@ -107,7 +119,17 @@ class axon:
         config.axon.maximum_concurrent_rpcs = maximum_concurrent_rpcs if maximum_concurrent_rpcs != None else config.axon.maximum_concurrent_rpcs
         config.axon.forward_timeout = forward_timeout if forward_timeout != None else config.axon.forward_timeout
         config.axon.backward_timeout = backward_timeout if backward_timeout != None else config.axon.backward_timeout
+        config.axon.compression = compression if compression != None else config.axon.compression
         axon.check_config( config )
+
+        # Determine the grpc compression algorithm
+        if config.axon.compression == 'gzip':
+            compress_alg = grpc.Compression.Gzip
+        elif config.axon.compression == 'deflate':
+            compress_alg = grpc.Compression.Deflate
+        else:
+            compress_alg = grpc.Compression.NoCompression
+
         if wallet == None:
             wallet = bittensor.wallet( config = config )
         if thread_pool == None:
@@ -116,28 +138,36 @@ class axon:
             server = grpc.server( thread_pool,
                                   interceptors=(AuthInterceptor(blacklist=blacklist),),
                                   maximum_concurrent_rpcs = config.axon.maximum_concurrent_rpcs,
+                                  options = [('grpc.keepalive_time_ms', 100000),
+                                             ('grpc.keepalive_timeout_ms', 500000)]
                                 )
 
-        forwards = [forward_text, forward_image, forward_tensor]
-        backwards = [backward_text, backward_image, backward_tensor]
+        synapses = {}
+        synapses[bittensor.proto.Synapse.SynapseType.TEXT_LAST_HIDDEN_STATE] = synapse_last_hidden
+        synapses[bittensor.proto.Synapse.SynapseType.TEXT_CAUSAL_LM] = synapse_causal_lm
+        synapses[bittensor.proto.Synapse.SynapseType.TEXT_CAUSAL_LM_NEXT] = synapse_causal_lm_next
+        synapses[bittensor.proto.Synapse.SynapseType.TEXT_SEQ_2_SEQ] = synapse_seq_2_seq
+        
+        synapse_check_function = synapse_checks if synapse_checks != None else axon.default_synapse_check
 
         if priority != None:
             priority_threadpool = bittensor.prioritythreadpool(config=config)
         else: 
             priority_threadpool = None
 
-
         axon_instance = axon_impl.Axon( 
             wallet = wallet, 
             server = server,
             ip = config.axon.ip,
             port = config.axon.port,
-            forwards = forwards,
-            backwards = backwards,
+            forward = forward_text,
+            backward = backward_text,
+            synapses = synapses,
+            synapse_checks = synapse_check_function,
             priority = priority,
             priority_threadpool = priority_threadpool,
-            forward_timeout = forward_timeout,
-            backward_timeout = backward_timeout
+            forward_timeout = config.axon.forward_timeout,
+            backward_timeout = config.axon.backward_timeout,
         )
         bittensor.grpc.add_BittensorServicer_to_server( axon_instance, server )
         full_address = str( config.axon.ip ) + ":" + str( config.axon.port )
@@ -153,48 +183,62 @@ class axon:
         axon.add_args( parser )
         return bittensor.config( parser )
 
+    @classmethod   
+    def help(cls):
+        """ Print help to stdout
+        """
+        parser = argparse.ArgumentParser()
+        cls.add_args( parser )
+        print (cls.__new__.__doc__)
+        parser.print_help()
+
     @classmethod
-    def add_args( cls, parser: argparse.ArgumentParser ):
+    def add_args( cls, parser: argparse.ArgumentParser, prefix: str = None  ):
         """ Accept specific arguments from parser
         """
+        prefix_str = '' if prefix == None else prefix + '.'
         try:
-            parser.add_argument('--axon.port', type=int, 
+            parser.add_argument('--' + prefix_str + 'axon.port', type=int, 
                     help='''The port this axon endpoint is served on. i.e. 8091''', default = bittensor.defaults.axon.port)
-            parser.add_argument('--axon.ip', type=str, 
+            parser.add_argument('--' + prefix_str + 'axon.ip', type=str, 
                 help='''The local ip this axon binds to. ie. [::]''', default = bittensor.defaults.axon.ip)
-            parser.add_argument('--axon.max_workers', type=int, 
+            parser.add_argument('--' + prefix_str + 'axon.max_workers', type=int, 
                 help='''The maximum number connection handler threads working simultaneously on this endpoint. 
                         The grpc server distributes new worker threads to service requests up to this number.''', default = bittensor.defaults.axon.max_workers)
-            parser.add_argument('--axon.maximum_concurrent_rpcs', type=int, 
+            parser.add_argument('--' + prefix_str + 'axon.maximum_concurrent_rpcs', type=int, 
                 help='''Maximum number of allowed active connections''',  default = bittensor.defaults.axon.maximum_concurrent_rpcs)
-            parser.add_argument('--axon.backward_timeout', type=int,
-                help='Number of seconds to wait for backward axon request', default=20)
-            parser.add_argument('--axon.forward_timeout', type=int,
-                help='Number of seconds to wait for forward axon request', default=10)
-            parser.add_argument('--axon.priority.max_workers', type = int,
+            parser.add_argument('--' + prefix_str + 'axon.backward_timeout', type=int,
+                help='Number of seconds to wait for backward axon request', default=2*bittensor.__blocktime__)
+            parser.add_argument('--' + prefix_str + 'axon.forward_timeout', type=int,
+                help='Number of seconds to wait for forward axon request', default=5*bittensor.__blocktime__)
+            parser.add_argument('--' + prefix_str + 'axon.priority.max_workers', type = int,
                 help='''maximum number of threads in thread pool''', default = bittensor.defaults.axon.priority.max_workers)
-            parser.add_argument('--axon.priority.maxsize', type=int, 
+            parser.add_argument('--' + prefix_str + 'axon.priority.maxsize', type=int, 
                 help='''maximum size of tasks in priority queue''', default = bittensor.defaults.axon.priority.maxsize)
+            parser.add_argument('--' + prefix_str + 'axon.compression', type=str, 
+                help='''Which compression algorithm to use for compression (gzip, deflate, NoCompression) ''', default = bittensor.defaults.axon.compression)
         except argparse.ArgumentError:
             # re-parsing arguments.
             pass
 
-        bittensor.wallet.add_args( parser )
+        bittensor.wallet.add_args( parser, prefix = prefix )
 
     @classmethod   
     def add_defaults(cls, defaults):
         """ Adds parser defaults to object from enviroment variables.
         """
-        defaults.axon = bittensor.Config()
+        defaults.axon = bittensor.config()
         defaults.axon.port = os.getenv('BT_AXON_PORT') if os.getenv('BT_AXON_PORT') != None else 8091
         defaults.axon.ip = os.getenv('BT_AXON_IP') if os.getenv('BT_AXON_IP') != None else '[::]'
         defaults.axon.max_workers = os.getenv('BT_AXON_MAX_WORERS') if os.getenv('BT_AXON_MAX_WORERS') != None else 10
         defaults.axon.maximum_concurrent_rpcs = os.getenv('BT_AXON_MAXIMUM_CONCURRENT_RPCS') if os.getenv('BT_AXON_MAXIMUM_CONCURRENT_RPCS') != None else 400
         
-        defaults.axon.priority = bittensor.Config()
+        defaults.axon.priority = bittensor.config()
         defaults.axon.priority.max_workers = os.getenv('BT_AXON_PRIORITY_MAX_WORKERS') if os.getenv('BT_AXON_PRIORITY_MAX_WORKERS') != None else 10
         defaults.axon.priority.maxsize = os.getenv('BT_AXON_PRIORITY_MAXSIZE') if os.getenv('BT_AXON_PRIORITY_MAXSIZE') != None else -1
 
+        defaults.axon.compression = 'NoCompression'
+
     @classmethod   
     def check_config(cls, config: 'bittensor.Config' ):
         """ Check config for axon port and wallet
@@ -202,56 +246,42 @@ class axon:
         assert config.axon.port > 1024 and config.axon.port < 65535, 'port must be in range [1024, 65535]'
         bittensor.wallet.check_config( config )
 
+    @classmethod   
+    def default_synapse_check(cls, synapse, hotkey ):
+        """ default synapse check function
+        """
+        if len(hotkey) == bittensor.__ss58_address_length__:
+            return True
+        
+        return False
+
     @staticmethod
-    def check_backward_callback( backward_callback:Callable, modality:int, pubkey:str = '_' ):
+    def check_backward_callback( backward_callback:Callable, pubkey:str = '_' ):
         """ Check and test axon backward callback function
         """
         if not inspect.ismethod(backward_callback) and not inspect.isfunction(backward_callback):
             raise ValueError('The axon backward callback must be a function with signature Callable[inputs_x:torch.FloatTensor, grads_dy:torch.FloatTensor ) -> torch.FloatTensor:, got {}'.format(backward_callback))        
-        if len( inspect.signature(backward_callback).parameters) != 2:
-            raise ValueError('The axon backward callback must have signature Callable[ inputs_x:torch.FloatTensor, grads_dy:torch.FloatTensor ) -> torch.FloatTensor:, got {}'.format(inspect.signature(backward_callback)))
+        if len( inspect.signature(backward_callback).parameters) != 3:
+            raise ValueError('The axon backward callback must have signature Callable[ inputs_x:torch.FloatTensor, grads_dy:torch.FloatTensor, synapses ) -> torch.FloatTensor:, got {}'.format(inspect.signature(backward_callback)))
         if 'inputs_x' not in inspect.signature(backward_callback).parameters:
             raise ValueError('The axon backward callback must have signature Callable[inputs_x:torch.FloatTensor, grads_dy:torch.FloatTensor ) -> torch.FloatTensor:, got {}'.format(inspect.signature(backward_callback)))
         if 'grads_dy' not in inspect.signature(backward_callback).parameters:
             raise ValueError('The axon backward callback must have signature Callable[inputs_x:torch.FloatTensor, grads_dy:torch.FloatTensor ) -> torch.FloatTensor:, got {}'.format(inspect.signature(backward_callback)))
  
-        if modality == bittensor.proto.Modality.TEXT:
-            sample_input = torch.randint(0,1,(3, 3))
-            grads_raw = torch.rand(3, 3, bittensor.__network_dim__)
-            backward_callback(sample_input,grads_raw)
-
-        if modality == bittensor.proto.Modality.IMAGE:
-            sample_input = torch.rand(1,1,3,512,512)
-            grads_raw = torch.rand(512, 512, bittensor.__network_dim__)
-            backward_callback(sample_input,grads_raw)
-
-        if modality == bittensor.proto.Modality.TENSOR:
-            sample_input = torch.rand(1,1,1)
-            grads_raw = torch.rand(1, 1, bittensor.__network_dim__)
-            backward_callback(sample_input,grads_raw)
 
     @staticmethod
-    def check_forward_callback( forward_callback:Callable, modality:int, pubkey:str = '_'):
+    def check_forward_callback( forward_callback:Callable, synapses:list = []):
         """ Check and test axon forward callback function
         """
         if not inspect.ismethod(forward_callback) and not inspect.isfunction(forward_callback):
             raise ValueError('The axon forward callback must be a function with signature Callable[inputs_x: torch.Tensor] -> torch.FloatTensor:, got {}'.format(forward_callback))   
-        if len( inspect.signature(forward_callback).parameters) != 1:
-            raise ValueError('The axon forward callback must have signature Callable[ inputs_x: torch.Tensor] -> torch.FloatTensor:, got {}'.format(inspect.signature(forward_callback)))
+        if len( inspect.signature(forward_callback).parameters) != 3:
+            raise ValueError('The axon forward callback must have signature Callable[ inputs_x: torch.Tensor, synapses, hotkey] -> torch.FloatTensor:, got {}'.format(inspect.signature(forward_callback)))
         if 'inputs_x' not in inspect.signature(forward_callback).parameters:
             raise ValueError('The axon forward callback must have signature Callable[ inputs_x: torch.Tensor] -> torch.FloatTensor:, got {}'.format(inspect.signature(forward_callback)))
         
-        if modality == bittensor.proto.Modality.TEXT:
-            sample_input = torch.randint(0,1,(3, 3))
-            forward_callback(sample_input)
-
-        if modality == bittensor.proto.Modality.IMAGE:
-            sample_input = torch.rand(1,1,3,512,512)
-            forward_callback(sample_input)
-            
-        if modality == bittensor.proto.Modality.TENSOR:
-            sample_input = torch.rand(1,1,1)
-            forward_callback(sample_input)
+        sample_input = torch.randint(0,1,(3, 3))
+        forward_callback([sample_input], synapses, hotkey='')
 
 class AuthInterceptor(grpc.ServerInterceptor):
     """ Creates a new server interceptor that authenticates incoming messages from passed arguments.
@@ -300,26 +330,28 @@ class AuthInterceptor(grpc.ServerInterceptor):
         r"""vertification of signature in metadata. Uses the pubkey and nounce
         """
         variable_length_messages = meta[1].value.split('bitxx')
-        nounce = variable_length_messages[0]
+        nounce = int(variable_length_messages[0])
         pubkey = variable_length_messages[1]
         message = variable_length_messages[2]
-        data_time = datetime.strptime(nounce,'%m%d%Y%H%M%S%f')
+        unique_receptor_uid = variable_length_messages[3]
         _keypair = Keypair(ss58_address=pubkey)
 
+        # Unique key that specifies the endpoint.
+        endpoint_key = str(pubkey) + str(unique_receptor_uid)
         
         #checking the time of creation, compared to previous messages
-        if pubkey in self.nounce_dic.keys():
-            prev_data_time = self.nounce_dic[pubkey]
-            if data_time - prev_data_time >= timedelta(milliseconds=1):
-                self.nounce_dic[pubkey] = data_time
+        if endpoint_key in self.nounce_dic.keys():
+            prev_data_time = self.nounce_dic[ endpoint_key ]
+            if nounce - prev_data_time > -10:
+                self.nounce_dic[ endpoint_key ] = nounce
 
                 #decrypting the message and verify that message is correct
-                verification = _keypair.verify(nounce+pubkey, message)
+                verification = _keypair.verify( str(nounce) + str(pubkey) + str(unique_receptor_uid), message)
             else:
                 verification = False
         else:
-            self.nounce_dic[pubkey] = data_time
-            verification = _keypair.verify(nounce+pubkey,message)
+            self.nounce_dic[ endpoint_key ] = nounce
+            verification = _keypair.verify( str( nounce ) + str(pubkey) + str(unique_receptor_uid), message)
 
         return verification
 
@@ -342,11 +374,11 @@ class AuthInterceptor(grpc.ServerInterceptor):
     def black_list_checking(self,meta):
         r"""Tries to call to blacklist function in the miner and checks if it should blacklist the pubkey 
         """
-        _, pubkey, _ = meta[1].value.split('bitxx')
-
+        variable_length_messages = meta[1].value.split('bitxx')
+        pubkey = variable_length_messages[1]
+        
         if self.blacklist == None:
             pass
-        #TODO: Turn on blacklisting
         elif self.blacklist(pubkey,int(meta[3].value)):
             raise Exception('Black listed')
         else:
