@@ -53,6 +53,19 @@ class logging:
             record_log: bool = None,
             logging_dir: str = None,
         ):
+        r""" Instantiate bittensor logging system backend.
+            Args:
+                config (:obj:`bittensor.Config`, `optional`): 
+                    bittensor.logging.config()
+                debug (:obj:`bool`, `optional`):
+                    Turn on debug.
+                trace (:obj:`bool`, `optional`):
+                    Turn on trace.
+                record_log (:obj:`bool`, `optional`):
+                    If true, logs are saved to loggind dir.
+                logging_dir (:obj:`str`, `optional`):
+                    Directory where logs are sunk. 
+        """   
 
         cls.__has_been_inited__ = True
 
@@ -115,15 +128,25 @@ class logging:
         logging.add_args( parser )
         return bittensor.config( parser )
 
+    @classmethod   
+    def help(cls):
+        """ Print help to stdout
+        """
+        parser = argparse.ArgumentParser()
+        cls.add_args( parser )
+        print (cls.__new__.__doc__)
+        parser.print_help()
+
     @classmethod
-    def add_args(cls, parser: argparse.ArgumentParser):
+    def add_args(cls, parser: argparse.ArgumentParser, prefix: str = None ):
         """ Accept specific arguments fro parser
         """
+        prefix_str = '' if prefix == None else prefix + '.'
         try:
-            parser.add_argument('--logging.debug', action='store_true', help='''Turn on bittensor debugging information''', default = bittensor.defaults.logging.debug )
-            parser.add_argument('--logging.trace', action='store_true', help='''Turn on bittensor trace level information''', default = bittensor.defaults.logging.trace )
-            parser.add_argument('--logging.record_log', action='store_true', help='''Turns on logging to file.''', default = bittensor.defaults.logging.record_log )
-            parser.add_argument('--logging.logging_dir', type=str, help='Logging default root directory.', default = bittensor.defaults.logging.logging_dir )
+            parser.add_argument('--' + prefix_str + 'logging.debug', action='store_true', help='''Turn on bittensor debugging information''', default = bittensor.defaults.logging.debug )
+            parser.add_argument('--' + prefix_str + 'logging.trace', action='store_true', help='''Turn on bittensor trace level information''', default = bittensor.defaults.logging.trace )
+            parser.add_argument('--' + prefix_str + 'logging.record_log', action='store_true', help='''Turns on logging to file.''', default = bittensor.defaults.logging.record_log )
+            parser.add_argument('--' + prefix_str + 'logging.logging_dir', type=str, help='Logging default root directory.', default = bittensor.defaults.logging.logging_dir )
         except argparse.ArgumentError:
             # re-parsing arguments.
             pass
@@ -190,7 +213,7 @@ class logging:
         """
         extra = record['extra']
         if 'rpc' in extra:
-            log_format = "<blue>{time:YYYY-MM-DD HH:mm:ss.SSS}</blue> | " + extra['code_str'] + " | {extra[prefix]} | {extra[direction]} | {extra[arrow]} | {extra[uid_str]} | {extra[inputs]} | {extra[call_time]} | {extra[key_str]} | {extra[rpc_message]} \n"
+            log_format = "<blue>{time:YYYY-MM-DD HH:mm:ss.SSS}</blue> | " + extra['code_str'] + " | {extra[prefix]} | {extra[direction]} | {extra[arrow]} | {extra[uid_str]} | {extra[inputs]} | {extra[call_time]} | {extra[key_str]} | {extra[rpc_message]} | {extra[synapse]} \n"
             return log_format
         elif 'receptor' in extra:
             log_format = "<blue>{time:YYYY-MM-DD HH:mm:ss.SSS}</blue> | " + extra['action'] + " | uid:{extra[uid]} | {extra[ip_str]} | hotkey:{extra[hotkey]} | coldkey:{extra[coldkey]} \n"
@@ -219,7 +242,20 @@ class logging:
 
 
     @classmethod
-    def rpc_log( cls, axon: bool, forward: bool, is_response: bool, code:int, call_time: float, pubkey: str, uid: int = None, inputs:list = None, outputs:list = None, message:str = ''):
+    def rpc_log( 
+                 cls, 
+                 axon: bool,
+                 forward: bool,
+                 is_response: bool,
+                 code:int,
+                 call_time: float, 
+                 pubkey: str, 
+                 uid: int = None, 
+                 inputs:list = None, 
+                 outputs:list = None, 
+                 message:str = '',
+                 synapse:'bittensor.Synapse' = None
+        ):
         """ Debug logging for the communication between endpoints with axon/dendrite 
         """
 
@@ -259,8 +295,24 @@ class logging:
             inputs = str(list(inputs)) if inputs != None else '[x]'
         inputs = inputs.center(15)
 
+        if synapse != None:
+            synapse = codes.code_to_synapse(synapse)
+
         rpc_message = message if message != None else 'None'
-        logger.debug( 'rpc', rpc=True, prefix=prefix, direction=direction, arrow=arrow, call_time = call_time_str, uid_str=uid_str, key_str=key_str, code_str=code_str, inputs = inputs, rpc_message = rpc_message)
+        logger.debug( 
+                    'rpc', 
+                    rpc=True, 
+                    prefix=prefix, 
+                    direction=direction, 
+                    arrow=arrow, 
+                    call_time = call_time_str, 
+                    uid_str=uid_str, 
+                    key_str=key_str, 
+                    code_str=code_str, 
+                    inputs = inputs, 
+                    rpc_message = rpc_message,
+                    synapse = synapse
+        )
 
 
     @classmethod
