@@ -22,8 +22,12 @@ import os
 from typing import List
 from loguru import logger
 
+import ast
+import pandas
 import torch.nn.functional as f
 import torch
+import pickle
+import json
 
 import bittensor
 import bittensor.utils.networking as net
@@ -86,6 +90,7 @@ class Metagraph( torch.nn.Module ):
         self.endpoints = torch.nn.Parameter( torch.tensor( [], dtype=torch.int64), requires_grad=False )
         self.uids = torch.nn.Parameter( torch.tensor([], dtype = torch.int64),requires_grad=False )
         self._endpoint_objs = None
+        self.neurons = None
         return self
 
     def forward (
@@ -277,11 +282,9 @@ class Metagraph( torch.nn.Module ):
         elif self._endpoint_objs != None:
             return self._endpoint_objs
         else:
+            self._endpoint_objs = []
             for tensor in self.endpoints:
-                try:
-                    obj = bittensor.endpoint.from_tensor( tensor )
-                except Exception:
-                    obj = None
+                obj = bittensor.endpoint.from_tensor( tensor )
                 self._endpoint_objs.append( obj )
             return self._endpoint_objs
 
@@ -377,16 +380,99 @@ class Metagraph( torch.nn.Module ):
         self._endpoint_objs = None
         return self
 
-    def sync ( self, block: int = None ) -> 'Metagraph':
+    def retrieve_cached_neurons( self, block: int = None ):
+        """
+            Retrieves cached metagraph syncs from IPFS. 
+        """
+        ipfs = bittensor.Ipfs()
+        ipns_hash = ipfs.latest_neurons_ipns
+        ipfs_hash = ipfs.cat
+        
+        if block != None:
+            ipns_hash = ipfs.historical_neurons_ipns
+            ipfs_hash = ipfs.node_get
+        
+        try:
+            # Ping IPNS for latest IPFS hash
+            ipns_resolve =  ipfs.retrieve_directory(ipfs.ipns_resolve, (('arg', ipns_hash),))
+
+            # Extract IPFS hash from IPNS response
+            ipfs_path = ast.literal_eval(ipns_resolve.text)
+        except Exception as e:
+            logger.error("Error detected in metagraph sync: {} with sample text {}".format(e,ipns_resolve.text))
+
+            # Try Again
+            # Ping IPNS for latest IPFS hash
+            ipns_resolve =  ipfs.retrieve_directory(ipfs.ipns_resolve, (('arg', ipns_hash),))
+
+            # Extract IPFS hash from IPNS response
+            ipfs_path = ast.literal_eval(ipns_resolve.text)
+
+        ipfs_resolved_hash = ipfs_path['Path'].split("ipfs/")[1]
+        ipfs_response = ipfs.retrieve_directory(ipfs_hash, (('arg', ipfs_resolved_hash),))
+
+        # Extract all neuron sync hashes
+        if block != None:
+            historical_neurons = json.loads(ipfs_response.content)['Links']
+            # Find the one that corresponds to our block
+            sync_data = next(item for item in historical_neurons if item["Name"] == "nakamoto-{}.pkl".format(block))
+            # Retrieve Neuron contents
+            ipfs_response = ipfs.retrieve_directory(ipfs.cat, (('arg', sync_data['Hash']),))
+
+        # Unpickle the response
+        neurons = pickle.loads(ipfs_response.content)
+
+        return neurons
+
+    def sync ( self, block: int = None, cached: bool = True ) -> 'Metagraph':
         r""" Synchronizes this metagraph with the chain state.
         """
+        logger.success(self.subtensor)
         if block == None:
             block = self.subtensor.get_current_block()
-            n_total = self.subtensor.get_n( )
-            neurons = self.subtensor.neurons()
+            if cached and self.subtensor.network in ("nakamoto", "local"):
+                if bittensor.__use_console__:
+                    with bittensor.__console__.status("Synchronizing Metagraph...", spinner="earth"):
+                        try:
+                            neurons = self.retrieve_cached_neurons( )
+                        except:
+                            # For some reason IPFS cache is down, fallback on regular sync
+                            logger.warning("IPFS cache may be down, falling back to regular sync")
+                            neurons = self.subtensor.neurons()
+                        n_total = len(neurons)
+                else:
+                    try:
+                        neurons = self.retrieve_cached_neurons( )
+                    except:
+                        # For some reason IPFS cache is down, fallback on regular sync
+                        logger.warning("IPFS cache may be down, falling back to regular sync")
+                        neurons = self.subtensor.neurons()
+                    n_total = len(neurons)
+            else:
+                neurons = self.subtensor.neurons( block = block )
+                n_total = len(neurons)
         else:
-            n_total = self.subtensor.get_n( block = block )
-            neurons = self.subtensor.neurons( block = block )
+            if cached and self.subtensor.network in ("nakamoto", "local"):
+                if bittensor.__use_console__:
+                    with bittensor.__console__.status("Synchronizing Metagraph...", spinner="earth"):
+                        try:
+                            neurons = self.retrieve_cached_neurons( block = block )
+                        except:
+                            # For some reason IPFS cache is down, fallback on regular sync
+                            logger.warning("IPFS cache may be down, falling back to regular sync to get block {}".format(block))
+                            neurons = self.subtensor.neurons( block = block )
+                        n_total = len(neurons)
+                else:
+                    try:
+                        neurons = self.retrieve_cached_neurons( block = block )
+                    except:
+                        # For some reason IPFS cache is down, fallback on regular sync
+                        logger.warning("IPFS cache may be down, falling back to regular sync to get block {}".format(block))
+                        neurons = self.subtensor.neurons( block = block )
+                    n_total = len(neurons)
+            else:
+                neurons = self.subtensor.neurons( block = block )
+                n_total = len(neurons)
 
         # Fill arrays.
         uids = [ i for i in range(n_total) ]
@@ -403,7 +489,9 @@ class Metagraph( torch.nn.Module ):
         weights = [ [ 0 for _ in range(n_total) ] for _ in range(n_total) ]
         bonds = [ [0 for _ in range(n_total) ] for _ in range(n_total) ]
         self._endpoint_objs = [ bittensor.endpoint.dummy() for _ in range(n_total) ]
+        self.neurons = [None for _ in range(n_total)]
         for n in neurons:
+            self.neurons[n.uid] = n
             uids[n.uid] = n.uid 
             active[n.uid] = n.active
             stake[n.uid] = n.stake 
@@ -476,9 +564,41 @@ class Metagraph( torch.nn.Module ):
             
         # For contructor.
         return self
-    
+
+    def to_dataframe(self):
+        try:
+            index = self.uids.tolist()
+            columns = [ 'uid', 'active', 'stake', 'rank', 'trust', 'consensus', 'incentive', 'dividends', 'emission']
+            dataframe = pandas.DataFrame(columns = columns, index = index)
+            for uid in self.uids.tolist():
+                v = {
+                    'uid': self.uids[uid].item(),
+                    'active': self.active[uid].item(),             
+                    'stake': self.stake[uid].item(),             
+                    'rank': self.ranks[uid].item(),            
+                    'trust': self.trust[uid].item(),             
+                    'consensus': self.consensus[uid].item(),             
+                    'incentive': self.incentive[uid].item(),             
+                    'dividends': self.dividends[uid].item(),             
+                    'emission': self.emission[uid].item()
+                }
+                dataframe.loc[uid] = pandas.Series( v )
+            dataframe['uid'] = dataframe.index
+            return dataframe
+        except Exception as e:
+            bittensor.logging.error('failed metagraph.to_dataframe()', str(e))
+            return pandas.DataFrame()
+
+    def to_wandb(self):
+        wandb_info = {
+            'metagraph_n': self.n.item(),
+            'metagraph_tau': self.tau.item(),
+            'metagraph_block': self.block.item(),
+        }
+        return wandb_info
+            
     def __str__(self):
         return "Metagraph({}, {}, {})".format(self.n.item(), self.block.item(), self.subtensor.network)
         
     def __repr__(self):
-        return self.__str__()
+        return self.__str__()
\ No newline at end of file
