@@ -18,26 +18,26 @@
 # DEALINGS IN THE SOFTWARE.
 
 import os
-import time
-import json
-import requests
+import sys
 from types import SimpleNamespace
+from typing import Optional, Union
 
-from typing import Union
+import bittensor
+from bittensor.utils import is_valid_bittensor_address_or_public_key
 from substrateinterface import Keypair
 from termcolor import colored
-import bittensor
 
-def display_mnemonic_msg( kepair : Keypair ):
+
+def display_mnemonic_msg( keypair : Keypair, key_type : str ):
     """ Displaying the mnemonic and warning message to keep mnemonic safe
     """
-    mnemonic = kepair.mnemonic
+    mnemonic = keypair.mnemonic
     mnemonic_green = colored(mnemonic, 'green')
     print (colored("\nIMPORTANT: Store this mnemonic in a secure (preferable offline place), as anyone " \
                 "who has possesion of this mnemonic can use it to regenerate the key and access your tokens. \n", "red"))
-    print ("The mnemonic to the new key is:\n\n%s\n" % mnemonic_green)
+    print ("The mnemonic to the new {} is:\n\n{}\n".format(key_type, mnemonic_green))
     print ("You can use the mnemonic to recreate the key in case it gets lost. The command to use to regenerate the key using this mnemonic is:")
-    print("bittensor-cli regen --mnemonic %s" % mnemonic)
+    print("btcli regen_{} --mnemonic {}".format(key_type, mnemonic))
     print('')
 
 class Wallet():
@@ -53,6 +53,7 @@ class Wallet():
         name:str,
         path:str,
         hotkey:str,
+        config: 'bittensor.Config' = None,
     ):
         r""" Init bittensor wallet object containing a hot and coldkey.
             Args:
@@ -62,6 +63,8 @@ class Wallet():
                     The name of hotkey used to running the miner.
                 path (required=True, default='~/.bittensor/wallets/'):
                     The path to your bittensor wallets
+                config (:obj:`bittensor.Config`, `optional`): 
+                    bittensor.wallet.config()
         """
         self.name = name
         self.path = path
@@ -69,6 +72,7 @@ class Wallet():
         self._hotkey = None
         self._coldkey = None
         self._coldkeypub = None
+        self.config = config
 
     def __str__(self):
         return "Wallet ({}, {}, {})".format(self.name, self.hotkey_str, self.path)
@@ -142,7 +146,7 @@ class Wallet():
                 is_registered (bool):
                     Is the wallet registered on the chain.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         return subtensor.is_hotkey_registered( self.hotkey.ss58_address )
 
     def get_neuron ( self, subtensor: 'bittensor.Subtensor' = None ) -> Union[ SimpleNamespace, None] :
@@ -154,7 +158,7 @@ class Wallet():
                 neuron (Union[ SimpleNamespace, None ]):
                     neuron account on the chain or None if you are not registered.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         if not self.is_registered(subtensor=subtensor): 
             print(colored('This wallet is not registered. Call wallet.register() before this function.','red'))
             return None
@@ -170,7 +174,7 @@ class Wallet():
                 uid (int):
                     Network uid or -1 if you are not registered.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         if not self.is_registered(subtensor=subtensor): 
             print(colored('This wallet is not registered. Call wallet.register() before this function.','red'))
             return -1
@@ -189,7 +193,7 @@ class Wallet():
                 balance (bittensor.utils.balance.Balance):
                     Stake account balance.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         if not self.is_registered(subtensor=subtensor): 
             print(colored('This wallet is not registered. Call wallet.register() before this function.','red'))
             return bittensor.Balance(0)
@@ -208,14 +212,46 @@ class Wallet():
                 balance (bittensor.utils.balance.Balance):
                     Coldkey balance.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         return subtensor.get_balance(address = self.coldkeypub.ss58_address)
 
+    def reregister(
+        self,
+        subtensor: 'bittensor.Subtensor' = None,
+        wait_for_inclusion: bool = False,
+        wait_for_finalization: bool = True,
+        prompt: bool = False
+    ) -> Optional['bittensor.Wallet']:
+        """ Re-register this wallet on the chain.
+            Args:
+                subtensor( 'bittensor.Subtensor' ):
+                    Bittensor subtensor connection. Overrides with defaults if None.
+                wait_for_inclusion (bool):
+                    if set, waits for the extrinsic to enter a block before returning true, 
+                    or returns false if the extrinsic fails to enter the block within the timeout.   
+                wait_for_finalization (bool):
+                    if set, waits for the extrinsic to be finalized on the chain before returning true,
+                    or returns false if the extrinsic fails to be finalized within the timeout.
+                prompt (bool):
+                    If true, the call waits for confirmation from the user before proceeding.
+                
+            Return:
+                wallet (bittensor.Wallet):
+                    This wallet.
+        """
+        if subtensor == None:
+            subtensor = bittensor.subtensor()
+        if not self.is_registered(subtensor=subtensor):
+            # Check if the wallet should reregister
+            if not self.config.wallet.get('reregister'):
+                sys.exit(0)
+        return self.register(subtensor=subtensor, wait_for_inclusion=wait_for_inclusion, wait_for_finalization=wait_for_finalization, prompt=prompt)
+
     def register ( 
             self, 
+            subtensor: 'bittensor.Subtensor' = None, 
             wait_for_inclusion: bool = False,
             wait_for_finalization: bool = True,
-            subtensor: 'bittensor.Subtensor' = None, 
             prompt: bool = False
         ) -> 'bittensor.Wallet':
         """ Registers this wallet on the chain.
@@ -234,7 +270,7 @@ class Wallet():
                 This wallet.
         """
         # Get chain connection.
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         subtensor.register( wallet = self, wait_for_inclusion = wait_for_inclusion, wait_for_finalization = wait_for_finalization, prompt=prompt )
         
         return self
@@ -265,7 +301,7 @@ class Wallet():
                     flag is true if extrinsic was finalized or uncluded in the block. 
                     If we did not wait for finalization / inclusion, the response is true.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         return subtensor.add_stake( wallet = self, amount = amount, wait_for_inclusion = wait_for_inclusion, wait_for_finalization = wait_for_finalization, prompt=prompt )
 
     def remove_stake( self, 
@@ -294,7 +330,7 @@ class Wallet():
                     flag is true if extrinsic was finalized or uncluded in the block. 
                     If we did not wait for finalization / inclusion, the response is true.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki')
+        if subtensor == None: subtensor = bittensor.subtensor()
         return subtensor.unstake( wallet = self, amount = amount, wait_for_inclusion = wait_for_inclusion, wait_for_finalization = wait_for_finalization, prompt=prompt )
 
     def transfer( 
@@ -328,7 +364,7 @@ class Wallet():
                     flag is true if extrinsic was finalized or uncluded in the block. 
                     If we did not wait for finalization / inclusion, the response is true.
         """
-        if subtensor == None: subtensor = bittensor.subtensor(network = 'akatsuki') 
+        if subtensor == None: subtensor = bittensor.subtensor() 
         return subtensor.transfer( wallet = self, dest = dest, amount = amount, wait_for_inclusion = wait_for_inclusion, wait_for_finalization = wait_for_finalization, prompt=prompt )
 
     def create_if_non_existent( self, coldkey_use_password:bool = True, hotkey_use_password:bool = False) -> 'Wallet':
@@ -356,15 +392,16 @@ class Wallet():
 
     @property
     def hotkey_file(self) -> 'bittensor.Keyfile':
+
         wallet_path = os.path.expanduser(os.path.join(self.path, self.name))
         hotkey_path = os.path.join(wallet_path, "hotkeys", self.hotkey_str)
-        return bittensor.Keyfile( path = hotkey_path )
+        return bittensor.keyfile( path = hotkey_path )
 
     @property
     def coldkey_file(self) -> 'bittensor.Keyfile':
         wallet_path = os.path.expanduser(os.path.join(self.path, self.name))
         coldkey_path = os.path.join(wallet_path, "coldkey")
-        return bittensor.Keyfile( path = coldkey_path )
+        return bittensor.keyfile( path = coldkey_path )
 
     @property
     def coldkeypub_file(self) -> 'bittensor.Keyfile':
@@ -449,7 +486,7 @@ class Wallet():
                     this object with newly created coldkey.
         """
         keypair = Keypair.create_from_uri( uri )
-        display_mnemonic_msg( keypair )
+        display_mnemonic_msg( keypair, "coldkey" )
         self.set_coldkey( keypair, encrypt = use_password, overwrite = overwrite)
         self.set_coldkeypub( keypair, overwrite = overwrite)
         return self
@@ -468,7 +505,7 @@ class Wallet():
                     this object with newly created hotkey.
         """
         keypair = Keypair.create_from_uri( uri )
-        display_mnemonic_msg( keypair  )
+        display_mnemonic_msg( keypair, "hotkey" )
         self.set_hotkey( keypair, encrypt=use_password, overwrite = overwrite)
         return self
 
@@ -502,7 +539,7 @@ class Wallet():
         """
         mnemonic = Keypair.generate_mnemonic( n_words)
         keypair = Keypair.create_from_mnemonic(mnemonic)
-        display_mnemonic_msg( keypair  )
+        display_mnemonic_msg( keypair, "coldkey" )
         self.set_coldkey( keypair, encrypt = use_password, overwrite = overwrite)
         self.set_coldkeypub( keypair, overwrite = overwrite)
         return self
@@ -537,15 +574,17 @@ class Wallet():
         """
         mnemonic = Keypair.generate_mnemonic( n_words)
         keypair = Keypair.create_from_mnemonic(mnemonic)
-        display_mnemonic_msg( keypair  )
+        display_mnemonic_msg( keypair, "hotkey" )
         self.set_hotkey( keypair, encrypt=use_password, overwrite = overwrite)
         return self
 
-    def regen_coldkey( self, mnemonic: Union[list, str], use_password: bool = True,  overwrite:bool = False) -> 'Wallet':
+    def regen_coldkey( self, mnemonic: Optional[Union[list, str]]=None, seed: Optional[str]=None, use_password: bool = True,  overwrite:bool = False) -> 'Wallet':
         """ Regenerates the coldkey from passed mnemonic, encrypts it with the user's password and save the file
             Args:
                 mnemonic: (Union[list, str], optional):
                     Key mnemonic as list of words or string space separated words.
+                seed: (str, optional):
+                    Seed as hex string.
                 use_password (bool, optional):
                     Is the created key password protected.
                 overwrite (bool, optional): 
@@ -554,13 +593,46 @@ class Wallet():
                 wallet (bittensor.Wallet):
                     this object with newly created coldkey.
         """
-        self.regenerate_coldkey(mnemonic, use_password, overwrite)
+        self.regenerate_coldkey(mnemonic, seed, use_password, overwrite)
 
-    def regenerate_coldkey( self, mnemonic: Union[list, str], use_password: bool = True,  overwrite:bool = False) -> 'Wallet':
+    def regenerate_coldkeypub( self, ss58_address: Optional[str] = None, public_key: Optional[Union[str, bytes]] = None, overwrite: bool = False ) -> 'Wallet':
+        """ Regenerates the coldkeypub from passed ss58_address or public_key and saves the file
+               Requires either ss58_address or public_key to be passed.
+            Args:
+                ss58_address: (str, optional):
+                    Address as ss58 string.
+                public_key: (str | bytes, optional):
+                    Public key as hex string or bytes.
+                overwrite (bool, optional) (default: False):
+                    Will this operation overwrite the coldkeypub (if exists) under the same path <wallet path>/<wallet name>/coldkeypub
+            Returns:
+                wallet (bittensor.Wallet):
+                    newly re-generated Wallet with coldkeypub.
+            
+        """
+        if ss58_address is None and public_key is None:
+            raise ValueError("Either ss58_address or public_key must be passed")
+
+        if not is_valid_bittensor_address_or_public_key( ss58_address if ss58_address is not None else public_key ):
+            raise ValueError(f"Invalid {'ss58_address' if ss58_address is not None else 'public_key'}") 
+
+        keypair = Keypair(ss58_address=ss58_address, public_key=public_key, ss58_format=bittensor.__ss58_format__)
+
+        # No need to encrypt the public key
+        self.set_coldkeypub( keypair, overwrite = overwrite)
+
+        return self
+
+    # Short name for regenerate_coldkeypub
+    regen_coldkeypub = regenerate_coldkeypub
+
+    def regenerate_coldkey( self, mnemonic: Optional[Union[list, str]]=None, seed: Optional[str]=None, use_password: bool = True,  overwrite:bool = False) -> 'Wallet':
         """ Regenerates the coldkey from passed mnemonic, encrypts it with the user's password and save the file
             Args:
                 mnemonic: (Union[list, str], optional):
                     Key mnemonic as list of words or string space separated words.
+                seed: (str, optional):
+                    Seed as hex string.
                 use_password (bool, optional):
                     Is the created key password protected.
                 overwrite (bool, optional): 
@@ -569,11 +641,18 @@ class Wallet():
                 wallet (bittensor.Wallet):
                     this object with newly created coldkey.
         """
-        if isinstance( mnemonic, str): mnemonic = mnemonic.split()
-        if len(mnemonic) not in [12,15,18,21,24]:
-            raise ValueError("Mnemonic has invalid size. This should be 12,15,18,21 or 24 words")
-        keypair = Keypair.create_from_mnemonic(" ".join(mnemonic))
-        display_mnemonic_msg( keypair  )
+        if mnemonic is None and seed is None:
+            raise ValueError("Must pass either mnemonic or seed")
+        if mnemonic is not None:
+            if isinstance( mnemonic, str): mnemonic = mnemonic.split()
+            if len(mnemonic) not in [12,15,18,21,24]:
+                raise ValueError("Mnemonic has invalid size. This should be 12,15,18,21 or 24 words")
+            keypair = Keypair.create_from_mnemonic(" ".join(mnemonic))   
+            display_mnemonic_msg( keypair, "coldkey" )
+        else:
+            # seed is not None
+            keypair = Keypair.create_from_seed(seed)
+            
         self.set_coldkey( keypair, encrypt = use_password, overwrite = overwrite)
         self.set_coldkeypub( keypair, overwrite = overwrite)
         return self 
@@ -610,6 +689,6 @@ class Wallet():
         if len(mnemonic) not in [12,15,18,21,24]:
             raise ValueError("Mnemonic has invalid size. This should be 12,15,18,21 or 24 words")
         keypair = Keypair.create_from_mnemonic(" ".join(mnemonic))
-        display_mnemonic_msg( keypair  )
+        display_mnemonic_msg( keypair, "hotkey" )
         self.set_hotkey( keypair, encrypt=use_password, overwrite = overwrite)
-        return self 
\ No newline at end of file
+        return self 
